@@ -2,10 +2,13 @@
 from __future__ import print_function
 
 import argparse
+import contextlib
 import hashlib
 import os
+import shutil
 import subprocess
 import sys
+import tempfile
 
 from binstar_client.utils import get_binstar
 import binstar_client.errors
@@ -16,6 +19,16 @@ from conda_build.build import bldpkg_path
 from conda_build.config import Config
 
 
+@contextlib.contextmanager
+def get_temp_token(token):
+    dn = tempfile.mkdtemp()
+    fn = os.path.join(dn, "binstar.token")
+    with open(fn, "w") as fh:
+        fh.write(token)
+    yield fn
+    shutil.rmtree(dn)
+
+
 def built_distribution_already_exists(cli, meta, owner):
     """
     Checks to see whether the built recipe (aka distribution) already
@@ -54,16 +67,12 @@ def built_distribution_already_exists(cli, meta, owner):
 
 
 def upload(cli, meta, owner, channels):
-    try:
-        with open('binstar.token', 'w') as fh:
-            fh.write(cli.token)
-        subprocess.check_call(['anaconda', '--quiet', '-t', 'binstar.token',
+    with get_temp_token(cli.token) as fn:
+        subprocess.check_call(['anaconda', '--quiet', '-t', fn,
                                'upload', bldpkg_path(meta),
                                '--user={}'.format(owner),
                                '--channel={}'.format(channels)],
                               env=os.environ)
-    finally:
-        os.remove('binstar.token')
 
 
 def distribution_exists_on_channel(binstar_cli, meta, owner, channel='main'):
@@ -103,22 +112,24 @@ def main():
     recipe_dir, owner, channel = args.recipe_dir, args.owner, args.channel
 
     cli = get_binstar(argparse.Namespace(token=token, site=None))
-    meta = MetaData(recipe_dir)
-    if meta.skip():
-        print("No upload to take place - this configuration was skipped in build/skip.")
-        return
-    exists = built_distribution_already_exists(cli, meta, owner)
-    if token:
-        if not exists:
-            upload(cli, meta, owner, channel)
-            print('Uploaded {}'.format(bldpkg_path(meta)))
+    meta_main = MetaData(recipe_dir)
+    for _, meta in meta_main.get_output_metadata_set(files=None):
+        print("Processing {}".format(meta.name()))
+        if meta.skip():
+            print("No upload to take place - this configuration was skipped in build/skip.")
+            continue
+        exists = built_distribution_already_exists(cli, meta, owner)
+        if token:
+            if not exists:
+                upload(cli, meta, owner, channel)
+                print('Uploaded {}'.format(bldpkg_path(meta)))
+            else:
+                print('Distribution {} already \nexists for {}.'
+                      ''.format(bldpkg_path(meta), owner))
         else:
-            print('Distribution {} already \nexists for {}.'
-                  ''.format(bldpkg_path(meta), owner))
-    else:
-        print("No BINSTAR_TOKEN present, so no upload is taking place. "
-              "The distribution just built {} already available for {}."
-              "".format('is' if exists else 'is not', owner))
+            print("No BINSTAR_TOKEN present, so no upload is taking place. "
+                  "The distribution just built {} already available for {}."
+                  "".format('is' if exists else 'is not', owner))
 
 if __name__ == '__main__':
     main()
